# remember base directory for sources of current lib/app
THIS_SRC_BASE_DIR:=$(CURDIR)
export THIS_SRC_BASE_DIR

include $(MAKEFILES_BASE_DIR)/common/dirs_features.mk


#
# find source files in all sub-directories
#

# find all sources selected by MODE
MODE_EXPRESSION :=$(shell cat $(THIS_SRC_FEATURES_MODES_DIR)/$(MODE))
ALL_MODE_SOURCES:=$(wildcard $(MODE_EXPRESSION))

ALL_SOURCES_C   :=$(filter %.c,   $(ALL_MODE_SOURCES))
ALL_SOURCES_CXX :=$(filter %.cpp, $(ALL_MODE_SOURCES))
ALL_SOURCES     :=$(ALL_SOURCES_C) $(ALL_SOURCES_CXX)
export ALL_SOURCES \
       ALL_SOURCES_C \
       ALL_SOURCES_CXX

CSRCS_TEST  :=$(filter %.t.c,   $(ALL_SOURCES_C)  )
CXXSRCS_TEST:=$(filter %.t.cpp, $(ALL_SOURCES_CXX))
export CSRCS_TEST \
       CXXSRCS_TEST

CSRCS_MTEST  :=$(filter %.mt.c,   $(ALL_SOURCES_C)  )
CXXSRCS_MTEST:=$(filter %.mt.cpp, $(ALL_SOURCES_CXX))
export CSRCS_MTEST \
       CXXSRCS_MTEST

CSRCS  :=$(filter-out $(CSRCS_TEST)   $(CSRCS_MTEST),   $(ALL_SOURCES_C)  )
CXXSRCS:=$(filter-out $(CXXSRCS_TEST) $(CXXSRCS_MTEST), $(ALL_SOURCES_CXX))
export CSRCS \
       CXXSRCS

ifneq (,$(CSRCS))
CSRCS_NOMAIN  :=$(shell grep -L '^int main' $(CSRCS)   )
endif # ALL_SOURCES_C
ifneq (,$(CXXSRCS))
CXXSRCS_NOMAIN:=$(shell grep -L '^int main' $(CXXSRCS) )
endif # ALL_SOURCES_CXX
export CSRCS_NOMAIN \
       CXXSRCS_NOMAIN

# directories containing source files
SOURCE_DIRS:=$(shell { for f in $(ALL_SOURCES) ; \
					  do dirname $$f; done; } | sort | uniq )
export SOURCE_DIRS


#
# make object names from source files
#

COBJS  :=$(CSRCS:.c=.o)
CXXOBJS:=$(CXXSRCS:.cpp=.o)
export COBJS \
       CXXOBJS

# obj files without 'main()':
COBJS_NOMAIN  :=$(CSRCS_NOMAIN:.c=.o)
CXXOBJS_NOMAIN:=$(CXXSRCS_NOMAIN:.cpp=.o)
export COBJS_NOMAIN \
       CXXOBJS_NOMAIN

# test object files:
COBJS_TEST  :=$(CSRCS_TEST:.c=.o)
CXXOBJS_TEST:=$(CXXSRCS_TEST:.cpp=.o)
export COBJS_TEST \
       CXXOBJS_TEST

# mtest object files:
COBJS_MTEST  :=$(CSRCS_MTEST:.c=.o)
CXXOBJS_MTEST:=$(CXXSRCS_MTEST:.cpp=.o)
export COBJS_MTEST \
       CXXOBJS_MTEST

# mtest output files:
CBIN_MTEST  :=$(CSRCS_MTEST:.c=)
CXXBIN_MTEST:=$(CXXSRCS_MTEST:.cpp=)
export CBIN_MTEST \
       CXXBIN_MTEST

# dep files:
CDEPS  :=$(CSRCS:.c=.d)     $(CSRCS_TEST:.c=.d)
CXXDEPS:=$(CXXSRCS:.cpp=.d) $(CXXSRCS_TEST:.cpp=.d)
export CDEPS \
       CXXDEPS

