/*
 * BasicTransforms.hpp
 *
 * templates for methods commonly used when working with
 * Point and Vector classes.
 *
 */
#ifndef INCLUDE_VECTORS_BASICTRANSFORMS_HPP_FILE
#define INCLUDE_VECTORS_BASICTRANSFORMS_HPP_FILE

/* public header */

#include "Vectors/Point.hpp"
#include "Vectors/Vector.hpp"
#include "Vectors/Exception.hpp"

namespace Vectors
{

// make point from vector
template<typename TPoint>
TPoint vector2point(const Vector<TPoint>& v)
{
  TPoint p;
  unsigned int dims=v.size();
  for(unsigned int i=0; i<dims; ++i)
    p[i]=v[i];
  return p;
} // vector2point()


// make vector from point
template<typename TPoint>
Vector<TPoint> point2vector(const TPoint& p)
{
  Vector<TPoint> v;
  unsigned int dims=p.size();
  for(unsigned int i=0; i<dims; ++i)
    v[i]=p[i];
  return v;
} // point2vector()


// move point by vector
template<typename TPoint>
TPoint movePointByVector(const TPoint& p, const Vector<TPoint>& v)
{
  TPoint       pOut;
  unsigned int dims=p.size();
  for(unsigned int i=0; i<dims; ++i)
    pOut[i]=p[i]+v[i];
  return pOut;
} // movePointByVector()

} // namespace Vectors

#endif

