/*
 * Line.hpp
 *
 * generic class representing single line beetween two
 * points. point type is given as the template argument.
 *
 * note: Point class MUST:
 *        -> beahave as a value
 *        -> implement / double getDist(const Point) const / method
 *
 */
#ifndef INCLUDE_VECTORS_LINE_HPP_FILE
#define INCLUDE_VECTORS_LINE_HPP_FILE

/* public header */

#include <boost/operators.hpp>

#include "Vectors/Point.hpp"
#include "Vectors/Vector.hpp"
#include "Vectors/BasicTransforms.hpp"
#include "Vectors/Exception.hpp"


namespace Vectors
{

template<typename TPoint>
class Line: public boost::equality_comparable< Line<TPoint> >
{
public:
  // make uninitialized line (for future usage)
  Line(void)
  {
  }

  // make Line starting in "from" point in direction and
  // length of "dir" Vector
  Line(const TPoint &from, const Vectors::Vector<TPoint> &dir):
    _from(from),
    _to  ( movePointByVector(from, dir) )
  {
  }

  // make line from 2 points
  Line(const TPoint &from, const TPoint &to):
    _from(from),
    _to  (to  )
  {
  }

  inline const TPoint &from(void) const
  {
    return _from;
  }

  inline const TPoint &to(void) const
  {
    return _to;
  }

  inline double length(void) const
  {
    return _from.dist(_to);
  }

  inline bool operator==(const Vectors::Line<TPoint> &l) const
  {
    return from()==l.from() && to()==l.to();
  }

private:
  TPoint _from;
  TPoint _to;
}; // class Line


//
// some commonly used types of lines
//

typedef Vectors::Line<Vectors::Point2DCont> Line2DCont;
typedef Vectors::Line<Vectors::Point2DDisc> Line2DDisc;


}; // namespace Vectors

#endif

