/*
 * Point.hpp
 *
 * generic class representing n-Dimentional point in
 * Euklidean space (continues or discret).
 *
 * each variable MUST beahave as a type (value)!
 *
 */
#ifndef INCLUDE_VECTORS_POINT_HPP_FILE
#define INCLUDE_VECTORS_POINT_HPP_FILE

/* public header */

#include <cmath>
#include <assert.h>
#include <boost/operators.hpp>

#include "Vectors/Exception.hpp"


namespace Vectors
{

// Tdims    - dimmentions size
// TdimType - each dimmention type
template<unsigned int Tdims, typename TDimType = double>
class Point: public boost::equality_comparable< Point<Tdims, TDimType> >
{
public:
  typedef TDimType                        TThisDimType;
  typedef Vectors::Point<Tdims, TDimType> TThisType;

  // return distance beetween two points
  inline double dist(const TThisType &p) const
  {
    double sum=0;
    for(unsigned int i=0; i<Tdims; ++i)
    {
      double di=_dim[i]-p._dim[i];
      sum+=di*di;
    }
    return ::sqrt(sum);
  }

  inline TDimType &operator[](unsigned int pos)
  {
    assert( pos<size() );
    return _dim[pos];
  }

  inline const TDimType &operator[](unsigned int pos) const
  {
    assert( pos<size() );
    return _dim[pos];
  }

  inline bool operator==(const TThisType &p) const
  {
    if( size()!=p.size() )      // size does not match?
      return false;

    for(unsigned int i=0; i<Tdims; ++i)
      if( (*this)[i]!=p[i] )
        return false;

    return true;
  }

  // returns dimmentions count
  inline unsigned int size(void) const
  {
    return Tdims;
  }

private:
  TDimType _dim[Tdims];
}; // class Point


//
// some commolny used spaces sizes allong with
// dimmention types
//

typedef Point<2, double> Point2DCont;
typedef Point<2, int   > Point2DDisc;

typedef Point<3, double> Point3DCont;
typedef Point<3, int   > Point3DDisc;

//
// operations for easyier creation of common
// point types
//

// make 2D point of any type
template<typename TDimType>
Point<2, TDimType> makePoint( const TDimType& x,
                              const TDimType& y )
{
  Point<2, TDimType> p;
  p[0]=x;
  p[1]=y;
  return p;
}

// make 3D point of any type
template<typename TDimType>
Point<3, TDimType> makePoint( const TDimType& x,
                              const TDimType& y,
                              const TDimType& z )
{
  Point<3, TDimType> p;
  p[0]=x;
  p[1]=y;
  p[2]=z;
  return p;
}


} // namespace Vectors


#endif

