/*
 * Point test set
 *
 */
#include <tut.h>

#include "Vectors/Point.hpp"

namespace Vectors
{

struct PointTestClass
{
  PointTestClass(void)
  {
    p1[0]=1;
    p1[1]=2;

    p2[0]=4;
    p2[1]=5;
  }

  Point<2> p1;
  Point<2> p2;
};

} // namespace Vectors


namespace tut
{
typedef Vectors::PointTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Vectors/Point");
}


using namespace Vectors;
const double eps = 0.001;

namespace tut
{

// default constructor test
template<>
template<>
void testObj::test<1>(void)
{
  Point<8> p4;
  Point<8, double> p5;

  p4[0]=p4[2];
  p4=p5;
}

// test assignment operator
template<>
template<>
void testObj::test<2>(void)
{
  Point<2, int> p;
  p[0]=10;
  p[1]=20;

  ensure( p[0]==10 );
  ensure( p[1]==20 );

  const Point<2, int> p3=p;
  ensure( p3[0]==p[0] );
  ensure( p3[1]==p[1] );
}

// test distance measure
template<>
template<>
void testObj::test<3>(void)
{
  ensure( fabs( p1.dist(p2) - 4.24264     ) < eps );
  ensure( fabs( p1.dist(p2) - p2.dist(p1) ) < eps );
}

// test comparison
template<>
template<>
void testObj::test<4>(void)
{
  ensure( p1!=p2 );

  p1=p2;
  ensure( p1==p2 );
}

// test size()
template<>
template<>
void testObj::test<5>(void)
{
  Point<8, float> p3;
  ensure( p3.size()==8 );
}

// test build of 2D point
template<>
template<>
void testObj::test<6>(void)
{
  Point2DCont p3=makePoint(10.0, 11.0);         // 2D case
  Point3DCont p4=makePoint(10.0, 11.0, 12.0);   // 3D case
}


} // namespace tut

