/*
 * Space.hpp
 *
 * template class representing space with some lines
 * int it, represented by abstract Line template class.
 *
 */
#ifndef INCLUDE_VECTORS_SPACE_HPP_FILE
#define INCLUDE_VECTORS_SPACE_HPP_FILE

/* public header */

#include <list>

#include "Vectors/Point.hpp"
#include "Vectors/Line.hpp"
#include "Vectors/Exception.hpp"



namespace Vectors
{

//
// space representation itself
//
template <typename TPoint>
class Space
{
public:
  // some useful typedefs
  typedef Line<TPoint>                              TLine;
  typedef typename std::list<TLine>::iterator       iterator;
  typedef typename std::list<TLine>::const_iterator const_iterator;

  inline unsigned int size(void) const
  {
    TPoint p;
    return p.size();
  }

  // collection I/O
  inline void add(const TLine &l)
  {
    _elems.insert( _elems.begin(), l);  // add at the begining
  }
  inline void remove(iterator &it)
  {
    _elems.erase(it);
  }
  // get number of elements in space (note: this call is O(n) !!)
  inline unsigned int count(void) const
  {
    return _elems.size();
  }

  // get iterators
  inline iterator begin(void)
  {
    return iterator( _elems.begin() );
  };
  inline iterator end(void)
  {
    return iterator( _elems.end()   );
  };

  // get const_iterators
  inline const_iterator begin(void) const
  {
    return const_iterator( _elems.begin() );
  };
  inline const_iterator end(void)   const
  {
    return const_iterator( _elems.end()   );
  };

protected:
  std::list<TLine> _elems;
}; // class Space


}; // namespace Vectors

#endif

