/*
 * test set form Space
 *
 */
#include <tut.h>

#include "Vectors/Space.hpp"

namespace Vectors
{

struct SpaceTestClass
{
  SpaceTestClass(void):
    _l1( makePoint(1.0, 3.0), makePoint(4.0, 8.0) ),
    _l2( makePoint(9.0, 4.0), makePoint(6.0, 1.0) )
  {
  }

  Line2DCont _l1;
  Line2DCont _l2;
  typedef Space<Point2DCont> TSpace;
  TSpace     _s;
};

} // namespace Vectors


namespace tut
{
typedef Vectors::SpaceTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Vectors/Space");
}


using namespace Vectors;
const double eps = 0.001;

namespace tut
{

// test size() member
template<>
template<>
void testObj::test<1>(void)
{
  Space< Point<14> > s;
  ensure( s.size()==14 );
}

// test begin() and end()
template<>
template<>
void testObj::test<2>(void)
{
  _s.add(_l1);
  _s.add(_l2);

  ensure( _s.count()==2 );

  TSpace::iterator b=_s.begin();    // one element...
  TSpace::iterator e=_s.end();      // ...and the second one
  --e;

  // we cannot compare stright away, since
  // order of elements is not defined
  ensure( (*b)!=(*e) );
  ensure( (*b)==_l1 || (*b)==_l2 );
  ensure( (*e)==_l1 || (*e)==_l2 );
}

// test case the same as in test<2> but for
// const iterators
template<>
template<>
void testObj::test<3>(void)
{
  _s.add(_l1);
  _s.add(_l2);

  const TSpace &sConst=_s;

  ensure( sConst.count()==2 );

  TSpace::const_iterator b=sConst.begin();  // one element...
  TSpace::const_iterator e=sConst.end();    // ...and the second one
  --e;

  // we cannot compare stright away, since
  // order of elements is not defined
  ensure( (*b)!=(*e) );
  ensure( (*b)==_l1 || (*b)==_l2 );
  ensure( (*e)==_l1 || (*e)==_l2 );
}

} // namespace tut

