/*
 * Vector test set
 *
 */
#include <tut.h>

#include "Vectors/Vector.hpp"

namespace Vectors
{

struct VectorTestClass
{
  VectorTestClass(void):
    v1( makePoint(1.0, 2.0), makePoint(4.0, 1.0) )
  {
    v2=v1;
  }

  Vector<Point2DCont> v1;
  Vector<Point2DCont> v2;
};

} // namespace Vectors


namespace tut
{
typedef Vectors::VectorTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Vectors/Vector");
}


using namespace Vectors;
const double eps = 0.001;

namespace tut
{

// make vector auto of 2 points
template<>
template<>
void testObj::test<1>(void)
{
  Point2DDisc  p1=makePoint(1, 2);
  Point2DDisc  p2=makePoint(3, 5);
  Vector2DDisc v( p1, p2 );
  ensure( v[0]==2 && v[1]==3 );
}

// check length of a vector
template<>
template<>
void testObj::test<2>(void)
{
  ensure( fabs( v1.length() - 3.162277 ) < eps );
}

// test adding 2 vectors
template<>
template<>
void testObj::test<3>(void)
{
  v2[1]=8;
  v1+=v2;
  ensure( v1[0]==6 );
  ensure( v1[1]==7 );
}

// test equality
template<>
template<>
void testObj::test<4>(void)
{
  ensure( v1==v2 );

  v2[1]=1234;
  ensure( v1!=v2 );
}

// test size()
template<>
template<>
void testObj::test<5>(void)
{
  Vector< Point<8> > v3;
  ensure( v3.size()==8 );
}

// test multiply by double and int
template<>
template<>
void testObj::test<6>(void)
{
  v1*=2;
  v2*=2.0;
  ensure( v1==v2 );
}

// test vor scalar multipliti
template<>
template<>
void testObj::test<7>(void)
{
  Vector2DCont v=Vector2DCont( makePoint(0.0, 0.0), makePoint(-1.0, -3.0) );
  ensure( fabs( v1*v-0.0 ) < eps );
}

// test normalization of vector
template<>
template<>
void testObj::test<8>(void)
{
  ensure( fabs(v2.length()-1.0)>eps );
  v2.normalize();
  ensure( fabs(v2.length()-1.0)<eps );
}

// test for reverse vector
template<>
template<>
void testObj::test<9>(void)
{
  v1.reverse();
  ensure( v1[0]==-v2[0] && v1[1]==-v2[1] );
}

} // namespace tut

